@@ -1,7 +1,14 @@
-from collections import defaultdict, OrderedDict
+from collections import defaultdict
 import datetime
 import json
 
+try:
+        from collections import OrderedDict
+except ImportError:
+        # python 2.6 or earlier, use backport
+        from ordereddict import OrderedDict
+
+
 from moto.core import BaseBackend
 from .comparisons import get_comparison_func
 from .utils import unix_time
@@ -36,7 +43,7 @@ class DynamoType(object):
         )
 
     def __repr__(self):
-        return "DynamoType: {}".format(self.to_json())
+        return "DynamoType: {0}".format(self.to_json())
 
     def to_json(self):
         return {self.type: self.value}
@@ -62,7 +69,7 @@ class Item(object):
             self.attrs[key] = DynamoType(value)
 
     def __repr__(self):
-        return "Item: {}".format(self.to_json())
+        return "Item: {0}".format(self.to_json())
 
     def to_json(self):
         attributes = {}
@@ -101,6 +108,10 @@ class Table(object):
         self.created_at = datetime.datetime.now()
         self.items = defaultdict(dict)
 
+    @property
+    def has_range_key(self):
+        return self.range_key_attr is not None
+
     @property
     def describe(self):
         results = {
@@ -122,7 +133,7 @@ class Table(object):
                 "TableSizeBytes": 0,
             }
         }
-        if self.range_key_attr:
+        if self.has_range_key:
             results["Table"]["KeySchema"]["RangeKeyElement"] = {
                 "AttributeName": self.range_key_attr,
                 "AttributeType": self.range_key_type
@@ -132,7 +143,7 @@ class Table(object):
     def __len__(self):
         count = 0
         for key, value in self.items.iteritems():
-            if self.range_key_attr:
+            if self.has_range_key:
                 count += len(value)
             else:
                 count += 1
@@ -143,7 +154,7 @@ class Table(object):
 
     def put_item(self, item_attrs):
         hash_value = DynamoType(item_attrs.get(self.hash_key_attr))
-        if self.range_key_attr:
+        if self.has_range_key:
             range_value = DynamoType(item_attrs.get(self.range_key_attr))
         else:
             range_value = None
@@ -157,6 +168,8 @@ class Table(object):
         return item
 
     def get_item(self, hash_key, range_key):
+        if self.has_range_key and not range_key:
+            raise ValueError("Table has a range key, but no range key was passed into get_item")
         try:
             if range_key:
                 return self.items[hash_key][range_key]
