@@ -1,25 +1,60 @@
+import copy
+import itertools
 from collections import defaultdict
 
 from boto.ec2.instance import Instance as BotoInstance, Reservation
 
 from moto.core import BaseBackend
+from .exceptions import InvalidIdError
 from .utils import (
     random_ami_id,
     random_instance_id,
     random_reservation_id,
     random_security_group_id,
     random_snapshot_id,
+    random_spot_request_id,
     random_subnet_id,
     random_volume_id,
     random_vpc_id,
+    random_eip_association_id,
+    random_eip_allocation_id,
+    random_ip,
 )
 
 
+class InstanceState(object):
+    def __init__(self, name='pending', code=0):
+        self.name = name
+        self.code = code
+
+
 class Instance(BotoInstance):
-    def __init__(self):
-        self._state_name = None
-        self._state_code = None
+    def __init__(self, image_id, user_data):
         super(Instance, self).__init__()
+        self.id = random_instance_id()
+        self.image_id = image_id
+        self._state = InstanceState("running", 16)
+        self.user_data = user_data
+
+    def start(self):
+        self._state.name = "running"
+        self._state.code = 16
+
+    def stop(self):
+        self._state.name = "stopped"
+        self._state.code = 80
+
+    def terminate(self):
+        self._state.name = "terminated"
+        self._state.code = 48
+
+    def reboot(self):
+        self._state.name = "running"
+        self._state.code = 16
+
+    def get_tags(self):
+        tags = ec2_backend.describe_tags(self.id)
+        return tags
 
 
 class InstanceBackend(object):
@@ -33,15 +68,14 @@ class InstanceBackend(object):
             if instance.id == instance_id:
                 return instance
 
-    def add_instances(self, image_id, count):
+    def add_instances(self, image_id, count, user_data):
         new_reservation = Reservation()
         new_reservation.id = random_reservation_id()
         for index in range(count):
-            new_instance = Instance()
-            new_instance.id = random_instance_id()
-            new_instance.image_id = image_id
-            new_instance._state_name = "pending"
-            new_instance._state_code = 0
+            new_instance = Instance(
+                image_id,
+                user_data,
+            )
             new_reservation.instances.append(new_instance)
         self.reservations[new_reservation.id] = new_reservation
         return new_reservation
@@ -50,8 +84,7 @@ class InstanceBackend(object):
         started_instances = []
         for instance in self.all_instances():
             if instance.id in instance_ids:
-                instance._state_name = "pending"
-                instance._state_code = 0
+                instance.start()
                 started_instances.append(instance)
 
         return started_instances
@@ -60,8 +93,7 @@ class InstanceBackend(object):
         stopped_instances = []
         for instance in self.all_instances():
             if instance.id in instance_ids:
-                instance._state_name = "stopping"
-                instance._state_code = 64
+                instance.stop()
                 stopped_instances.append(instance)
 
         return stopped_instances
@@ -70,8 +102,7 @@ class InstanceBackend(object):
         terminated_instances = []
         for instance in self.all_instances():
             if instance.id in instance_ids:
-                instance._state_name = "shutting-down"
-                instance._state_code = 32
+                instance.terminate()
                 terminated_instances.append(instance)
 
         return terminated_instances
@@ -80,9 +111,7 @@ class InstanceBackend(object):
         rebooted_instances = []
         for instance in self.all_instances():
             if instance.id in instance_ids:
-                # TODO double check instances go to pending when reboot
-                instance._state_name = "pending"
-                instance._state_code = 0
+                instance.reboot()
                 rebooted_instances.append(instance)
 
         return rebooted_instances
@@ -104,8 +133,32 @@ class InstanceBackend(object):
                 instances.append(instance)
         return instances
 
-    def all_reservations(self):
-        return self.reservations.values()
+    def get_reservations_by_instance_ids(self, instance_ids):
+        """ Go through all of the reservations and filter to only return those
+        associated with the given instance_ids.
+        """
+        reservations = []
+        for reservation in self.all_reservations(make_copy=True):
+            reservation_instance_ids = [instance.id for instance in reservation.instances]
+            matching_reservation = any(instance_id in reservation_instance_ids for instance_id in instance_ids)
+            if matching_reservation:
+                # We need to make a copy of the reservation because we have to modify the
+                # instances to limit to those requested
+                reservation.instances = [instance for instance in reservation.instances if instance.id in instance_ids]
+                reservations.append(reservation)
+        found_instance_ids = [instance.id for reservation in reservations for instance in reservation.instances]
+        if len(found_instance_ids) != len(instance_ids):
+            invalid_id = list(set(instance_ids).difference(set(found_instance_ids)))[0]
+            raise InvalidIdError(invalid_id)
+        return reservations
+
+    def all_reservations(self, make_copy=False):
+        if make_copy:
+            # Return copies so that other functions can modify them with changing
+            # the originals
+            return [copy.deepcopy(reservation) for reservation in self.reservations.values()]
+        else:
+            return [reservation for reservation in self.reservations.values()]
 
 
 class TagBackend(object):
@@ -121,18 +174,21 @@ class TagBackend(object):
     def delete_tag(self, resource_id, key):
         return self.tags[resource_id].pop(key)
 
-    def describe_tags(self):
+    def describe_tags(self, filter_resource_ids=None):
         results = []
         for resource_id, tags in self.tags.iteritems():
             ami = 'ami' in resource_id
             for key, value in tags.iteritems():
-                result = {
-                    'resource_id': resource_id,
-                    'key': key,
-                    'value': value,
-                    'resource_type': 'image' if ami else 'instance',
-                }
-                results.append(result)
+                if not filter_resource_ids or resource_id in filter_resource_ids:
+                    # If we're not filtering, or we are filtering and this
+                    # resource id is in the filter list, add this tag
+                    result = {
+                        'resource_id': resource_id,
+                        'key': key,
+                        'value': value,
+                        'resource_type': 'image' if ami else 'instance',
+                    }
+                    results.append(result)
         return results
 
 
@@ -163,8 +219,12 @@ class AmiBackend(object):
         self.amis[ami_id] = ami
         return ami
 
-    def describe_images(self):
-        return self.amis.values()
+    def describe_images(self, ami_ids=None):
+        if ami_ids:
+            images = [image for image in self.amis.values() if image.id in ami_ids]
+        else:
+            images = self.amis.values()
+        return images
 
     def deregister_image(self, ami_id):
         if ami_id in self.amis:
@@ -228,7 +288,7 @@ class SecurityRule(object):
 
     @property
     def unique_representation(self):
-        return "{}-{}-{}-{}-{}".format(
+        return "{0}-{1}-{2}-{3}-{4}".format(
                self.ip_protocol,
                self.from_port,
                self.to_port,
@@ -241,61 +301,72 @@ class SecurityRule(object):
 
 
 class SecurityGroup(object):
-    def __init__(self, group_id, name, description):
+    def __init__(self, group_id, name, description, vpc_id=None):
         self.id = group_id
         self.name = name
         self.description = description
         self.ingress_rules = []
         self.egress_rules = []
+        self.vpc_id = vpc_id
 
 
 class SecurityGroupBackend(object):
 
     def __init__(self):
-        self.groups = {}
+        # the key in the dict group is the vpc_id or None (non-vpc)
+        self.groups = defaultdict(dict)
         super(SecurityGroupBackend, self).__init__()
 
-    def create_security_group(self, name, description):
+    def create_security_group(self, name, description, vpc_id=None, force=False):
         group_id = random_security_group_id()
-        existing_group = self.get_security_group_from_name(name)
-        if existing_group:
-            return None
-        group = SecurityGroup(group_id, name, description)
-        self.groups[group_id] = group
+        if not force:
+            existing_group = self.get_security_group_from_name(name, vpc_id)
+            if existing_group:
+                return None
+        group = SecurityGroup(group_id, name, description, vpc_id=vpc_id)
+
+        self.groups[vpc_id][group_id] = group
         return group
 
     def describe_security_groups(self):
-        return self.groups.values()
+        return itertools.chain(*[x.values() for x in self.groups.values()])
 
-    def delete_security_group(self, name_or_group_id):
-        if name_or_group_id in self.groups:
-            # Group Id
-            return self.groups.pop(name_or_group_id)
-        else:
-            # Group Name
-            group = self.get_security_group_from_name(name_or_group_id)
+    def delete_security_group(self, name=None, group_id=None):
+        if group_id:
+            # loop over all the SGs, find the right one
+            for vpc in self.groups.values():
+                if group_id in vpc:
+                    return vpc.pop(group_id)
+        elif name:
+            # Group Name.  Has to be in standard EC2, VPC needs to be identified by group_id
+            group = self.get_security_group_from_name(name, None)
             if group:
-                return self.groups.pop(group.id)
+                return self.groups[None].pop(group.id)
 
-    def get_security_group_from_name(self, name):
-        for group_id, group in self.groups.iteritems():
+    def get_security_group_from_name(self, name, vpc_id):
+        for group_id, group in self.groups[vpc_id].iteritems():
             if group.name == name:
                 return group
 
-    def authorize_security_group_ingress(self, group_name, ip_protocol, from_port, to_port, ip_ranges=None, source_group_names=None):
-        group = self.get_security_group_from_name(group_name)
+        if name == 'default':
+            # If the request is for the default group and it does not exist, create it
+            default_group = ec2_backend.create_security_group("default", "The default security group", force=True)
+            return default_group
+
+    def authorize_security_group_ingress(self, group_name, ip_protocol, from_port, to_port, ip_ranges=None, source_group_names=None, vpc_id=None):
+        group = self.get_security_group_from_name(group_name, vpc_id)
         source_groups = []
         for source_group_name in source_group_names:
-            source_groups.append(self.get_security_group_from_name(source_group_name))
+            source_groups.append(self.get_security_group_from_name(source_group_name, vpc_id))
 
         security_rule = SecurityRule(ip_protocol, from_port, to_port, ip_ranges, source_groups)
         group.ingress_rules.append(security_rule)
 
-    def revoke_security_group_ingress(self, group_name, ip_protocol, from_port, to_port, ip_ranges=None, source_group_names=None):
-        group = self.get_security_group_from_name(group_name)
+    def revoke_security_group_ingress(self, group_name, ip_protocol, from_port, to_port, ip_ranges=None, source_group_names=None, vpc_id=None):
+        group = self.get_security_group_from_name(group_name, vpc_id)
         source_groups = []
         for source_group_name in source_group_names:
-            source_groups.append(self.get_security_group_from_name(source_group_name))
+            source_groups.append(self.get_security_group_from_name(source_group_name, vpc_id))
 
         security_rule = SecurityRule(ip_protocol, from_port, to_port, ip_ranges, source_groups)
         if security_rule in group.ingress_rules:
@@ -445,9 +516,160 @@ class SubnetBackend(object):
         return self.subnets.pop(subnet_id, None)
 
 
+class SpotInstanceRequest(object):
+    def __init__(self, spot_request_id, price, image_id, type, valid_from,
+                 valid_until, launch_group, availability_zone_group, key_name,
+                 security_groups, user_data, instance_type, placement, kernel_id,
+                 ramdisk_id, monitoring_enabled, subnet_id):
+        self.id = spot_request_id
+        self.state = "open"
+        self.price = price
+        self.image_id = image_id
+        self.type = type
+        self.valid_from = valid_from
+        self.valid_until = valid_until
+        self.launch_group = launch_group
+        self.availability_zone_group = availability_zone_group
+        self.key_name = key_name
+        self.user_data = user_data
+        self.instance_type = instance_type
+        self.placement = placement
+        self.kernel_id = kernel_id
+        self.ramdisk_id = ramdisk_id
+        self.monitoring_enabled = monitoring_enabled
+        self.subnet_id = subnet_id
+
+        self.security_groups = []
+        if security_groups:
+            for group_name in security_groups:
+                group = ec2_backend.get_security_group_from_name(group_name, None)
+                if group:
+                    self.security_groups.append(group)
+        else:
+            # If not security groups, add the default
+            default_group = ec2_backend.get_security_group_from_name("default", None)
+            self.security_groups.append(default_group)
+
+
+class SpotRequestBackend(object):
+    def __init__(self):
+        self.spot_instance_requests = {}
+        super(SpotRequestBackend, self).__init__()
+
+    def request_spot_instances(self, price, image_id, count, type, valid_from,
+                               valid_until, launch_group, availability_zone_group,
+                               key_name, security_groups, user_data,
+                               instance_type, placement, kernel_id, ramdisk_id,
+                               monitoring_enabled, subnet_id):
+        requests = []
+        for index in range(count):
+            spot_request_id = random_spot_request_id()
+            request = SpotInstanceRequest(
+                spot_request_id, price, image_id, type, valid_from, valid_until,
+                launch_group, availability_zone_group, key_name, security_groups,
+                user_data, instance_type, placement, kernel_id, ramdisk_id,
+                monitoring_enabled, subnet_id
+            )
+            self.spot_instance_requests[spot_request_id] = request
+            requests.append(request)
+        return requests
+
+    def describe_spot_instance_requests(self):
+        return self.spot_instance_requests.values()
+
+    def cancel_spot_instance_requests(self, request_ids):
+        requests = []
+        for request_id in request_ids:
+            requests.append(self.spot_instance_requests.pop(request_id))
+        return requests
+
+
+class ElasticAddress():
+    def __init__(self, domain):
+        self.public_ip = random_ip()
+        self.allocation_id = random_eip_allocation_id() if domain == "vpc" else None
+        self.domain = domain
+        self.instance = None
+        self.association_id = None
+
+
+class ElasticAddressBackend(object):
+
+    def __init__(self):
+        self.addresses = []
+        super(ElasticAddressBackend, self).__init__()
+
+    def allocate_address(self, domain):
+        address = ElasticAddress(domain)
+        self.addresses.append(address)
+        return address
+
+    def address_by_ip(self, ips):
+        return [address for address in self.addresses
+                if address.public_ip in ips]
+
+    def address_by_allocation(self, allocation_ids):
+        return [address for address in self.addresses
+                if address.allocation_id in allocation_ids]
+
+    def address_by_association(self, association_ids):
+        return [address for address in self.addresses
+                if address.association_id in association_ids]
+
+    def associate_address(self, instance, address=None, allocation_id=None, reassociate=False):
+        eips = []
+        if address:
+            eips = self.address_by_ip([address])
+        elif allocation_id:
+            eips = self.address_by_allocation([allocation_id])
+        eip = eips[0] if len(eips) > 0 else None
+
+        if eip and eip.instance is None or reassociate:
+            eip.instance = instance
+            if eip.domain == "vpc":
+                eip.association_id = random_eip_association_id()
+            return eip
+        else:
+            return None
+
+    def describe_addresses(self):
+        return self.addresses
+
+    def disassociate_address(self, address=None, association_id=None):
+        eips = []
+        if address:
+            eips = self.address_by_ip([address])
+        elif association_id:
+            eips = self.address_by_association([association_id])
+
+        if eips:
+            eip = eips[0]
+            eip.instance = None
+            eip.association_id = None
+            return True
+        else:
+            return False
+
+    def release_address(self, address=None, allocation_id=None):
+        eips = []
+        if address:
+            eips = self.address_by_ip([address])
+        elif allocation_id:
+            eips = self.address_by_allocation([allocation_id])
+
+        if eips:
+            eip = eips[0]
+            self.disassociate_address(address=eip.public_ip)
+            eip.allocation_id = None
+            self.addresses.remove(eip)
+            return True
+        else:
+            return False
+
+
 class EC2Backend(BaseBackend, InstanceBackend, TagBackend, AmiBackend,
                  RegionsAndZonesBackend, SecurityGroupBackend, EBSBackend,
-                 VPCBackend, SubnetBackend):
+                 VPCBackend, SubnetBackend, SpotRequestBackend, ElasticAddressBackend):
     pass
 
 
