@@ -1,7 +1,6 @@
 from jinja2 import Template
 
 from moto.ec2.models import ec2_backend
-from moto.ec2.utils import resource_ids_from_querystring
 
 
 def process_rules_from_querystring(querystring):
@@ -22,9 +21,6 @@ def process_rules_from_querystring(querystring):
 
 
 class SecurityGroups(object):
-    def __init__(self, querystring):
-        self.querystring = querystring
-
     def authorize_security_group_egress(self):
         raise NotImplementedError('SecurityGroups.authorize_security_group_egress is not yet implemented')
 
@@ -35,21 +31,29 @@ class SecurityGroups(object):
     def create_security_group(self):
         name = self.querystring.get('GroupName')[0]
         description = self.querystring.get('GroupDescription')[0]
-        group = ec2_backend.create_security_group(name, description)
+        vpc_id = self.querystring.get("VpcId", [None])[0]
+        group = ec2_backend.create_security_group(name, description, vpc_id=vpc_id)
         if not group:
             # There was an exisitng group
-            return "There was an existing security group with name {}".format(name), dict(status=409)
+            return "There was an existing security group with name {0}".format(name), dict(status=409)
         template = Template(CREATE_SECURITY_GROUP_RESPONSE)
         return template.render(group=group)
 
     def delete_security_group(self):
         # TODO this should raise an error if there are instances in the group. See http://docs.aws.amazon.com/AWSEC2/latest/APIReference/ApiReference-query-DeleteSecurityGroup.html
-        name = self.querystring.get('GroupName')[0]
-        group = ec2_backend.delete_security_group(name)
 
+        name = self.querystring.get('GroupName')
+        sg_id = self.querystring.get('GroupId')
+
+        if name:
+            group = ec2_backend.delete_security_group(name[0])
+        elif sg_id:
+            group = ec2_backend.delete_security_group(group_id=sg_id[0])
+
+        # needs name or group now
         if not group:
             # There was no such group
-            return "There was no security group with name {}".format(name), dict(status=404)
+            return "There was no security group with name {0}".format(name), dict(status=404)
         return DELETE_GROUP_RESPONSE
 
     def describe_security_groups(self):
@@ -87,7 +91,7 @@ DESCRIBE_SECURITY_GROUPS_RESPONSE = """<DescribeSecurityGroupsResponse xmlns="ht
              <groupId>{{ group.id }}</groupId>
              <groupName>{{ group.name }}</groupName>
              <groupDescription>{{ group.description }}</groupDescription>
-             <vpcId/>
+             <vpcId>{{ group.vpc_id or ""}}</vpcId>
              <ipPermissions>
                {% for rule in group.ingress_rules %}
                     <item>
