@@ -2,7 +2,7 @@ from .responses import EC2Response
 
 
 url_bases = [
-    "https?://ec2.us-east-1.amazonaws.com",
+    "https?://ec2.(.+).amazonaws.com",
 ]
 
 url_paths = {
